﻿define([
	// dojo
	"dojo",
	"dojo/_base/array",
	"dojo/_base/declare",
	"dojo/_base/event",
	"dojo/_base/lang",
	"dojo/dom-class",
	"dojo/on",
	"dojo/when",
	"dojo/aspect",
	// dijit
	"dijit/form/Select",
	"dijit/_WidgetBase",
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dijit/_CssStateMixin",
	// epi
	"epi-cms/component/MediaViewModel",
	"epi-cms/component/Media",
	"epi/dependency",
	"epi/epi",
	//"epi-cms/_ContentContextMixin",
	//"epi-cms/widget/_FolderTreeNode",

	"digizuite/command/CropCommand",
    "digizuite/component/MultiSelectList",
    "digizuite/component/SingleSelectList",

	// template
	"dojo/text!./ListTemplate.html",
	"digizuite/integration/ContentList"
], function (
	// dojo
	dojo,
	array,
	declare,
	event,
	lang,
	domClass,
	on,
	when,
	aspect,
	// dijit
	Select,
	_WidgetBase,
	_Widget,
	_TemplatedMixin,
	_WidgetsInTemplateMixin,
	_CssStateMixin,
	// epi
	MediaViewModel,
	mediaComponent,
	dependency,
	epi,
	//_ContentContextMixin,
	//_FolderTreeNode,
	// template
	CropCommand,
    MultiSelectList,
    SingleSelectList,
	Template
) {
		var DigiViewModel = declare([MediaViewModel], {
			storeKey: "digi.cms.content.light",
			postscript() {
                this.inherited(arguments);                                

				var settings = {
					category: "context",
					model: this.treeStoreModel,
					selection: this.selection,
					clipboard: this.clipboardManager
                };

                // remove Open In Image Editor command
				var editImageCmd = this.getCommand("editImage");
				if (editImageCmd) {
					var indexOfEditImage = -1;
					for (var i = 0; i < this.commands.length; ++i) {
						if (this.commands[i] === editImageCmd) {
							indexOfEditImage = i;
							break;
						}
					}
					if (indexOfEditImage >= 0) {
						this.commands.splice(indexOfEditImage, 1);
					}
                }

                // remove New Foder command with Digizuite provider
                var newFolderCmd = this.getCommand("episerver.core.contentfolder");
                if (newFolderCmd) {
                    var indexOfNewFolder = -1;
                    for (i = 0; i < this.commands.length; ++i) {
                        if (this.commands[i] === newFolderCmd) {
                            indexOfNewFolder = i;
                            break;
                        }
                    }
                    if (indexOfNewFolder >= 0) {
                        this.commands.splice(indexOfNewFolder, 1);
                    }
                }

				// add Crops command
				var command = new CropCommand(settings);
                this.commands.push(command);                
            }
		});
		return declare("digizuite/integration/DigiViewer", [mediaComponent], {

			templateString: Template,
			showCreateContentArea: false,
            searchStore: null,
            translationStore: null,
			hierarchicalListClass: "epi-mediaList digi-media-list",
			modelClassName: DigiViewModel,
            selectControls: [],

			postCreate: function () {                
                this.inherited(arguments);

				var registry = dependency.resolve("epi.storeregistry");
                this.searchStore = this.searchStore || registry.get("digizuite.widgetsearch");
                this.translationStore = this.translationStore || registry.get("digizuite.translation");

                this.getDigiZuiteConfigs();

                domClass.toggle(this.btnClearFilter.domNode, "dijitHidden", true);
                when(this.translationStore.executeMethod('GetTranslationsOnDemand', '?key=' + '/digizuite/generalcaptions/clear'),
                    function (response) {
                        this.btnClearFilter.set("label", response);
                        this.resize();
                    }.bind(this));

				this.own(aspect.around(this.model, "_createListChildrenQuery", function (originalMethod) {
					return function (method, args) {
						var query = originalMethod(method, args);
						query.query = "GetChildren";

						var types = this.typeSelect.value;
						if (types) {
							query.assetTypes = types;
						}

						var crops = this.cropSelect.value;
						if (crops) {
							query.crops = crops;
						}

						var sort = this.sortSelect.value;
						if (sort) {
							query.digiSort = sort;
                        }

                        if (query.assetTypes === undefined && query.crops === undefined && query.digiSort === undefined) {
                            domClass.toggle(this.btnClearFilter.domNode, "dijitHidden", true);
                        } else {
                            domClass.toggle(this.btnClearFilter.domNode, "dijitHidden", false);
                        }

						return query;

					}.bind(this);
				}.bind(this)));

				this.own(aspect.around(this.model.treeStoreModel, "getAncestors", function (originalMethod) {
					return function (currentContent, callback) {
						var treePaths = this.model.get("treePaths");

						var selectedFolder = this.tree.lastFocused;
						if (selectedFolder && selectedFolder.item) {
							return treePaths;
						}

						return originalMethod.apply(this.model.treeStoreModel, arguments);
					}.bind(this);
				}.bind(this)));
				
			},

			startup: function () {			
				this.invokeSearchManually = false;

				this.own(on(this.searchBox, 'searchBoxChange', function (evt) {
					var searchBoxValue = this.searchBox.textbox.value;

					if (!this.invokeSearchManually) {
						//This is a tricky, if the evt has value and contains prefixSearch,
						///donot set value of dropdown, it makes loop forever by change events
                        
                        if (searchBoxValue === "") {
                            this.resetFilterSelection();
                        } else {
                            var filters = searchBoxValue.split(" ");
                            this.selectControls.forEach(function (item) {
                                //find relevant dropdown and set with value
                                var itemFound = false;
                                filters.forEach(function (filterItem) {
                                    var filter = filterItem.split("=");
                                    if (filter.length === 2) {
                                        if (filter[0] === item.key) {
                                            item.controlType.set("value", filter[1]);
                                            itemFound = true;
                                        }
                                    }
                                });

                                //Clear not found dropdown from facet
                                if (!itemFound) {
                                    item.controlType.set("value", "");
                                }
                            });
                        }						
					}

					this.invokeSearchManually = false;
				}.bind(this)));

				on(this.typeSelect, "change", function (types) {
					// The crop should be visible if media or nothing is selected
                    var isCropVisible = types.length === 0 || types.find(function (type) { return type === 4; });
                    domClass.toggle(this.cropSelect.domNode, "dijitHidden", !isCropVisible);

                    this.model._updateListQuery(this.model.selectedTreeItems);
				}.bind(this));
				on(this.sortSelect, "change", function (evt) {
					this.model._updateListQuery(this.model.selectedTreeItems);
				}.bind(this));
				on(this.cropSelect, "change", function (evt) {
					this.model._updateListQuery(this.model.selectedTreeItems);
				}.bind(this));

				this.inherited(arguments);

				this.submitSearch = function () {
					var term = this.searchBox.textbox.value;
					term = term.replace(/type\=[a-zA-Z]+/, '');
					term = term.replace(/sort\=[a-zA-Z]+/, '');
					term = term.replace(/crop\=[a-zA-Z]+/, '');
					term = $.trim(term);
					if (this.typeSelect.value !== '') {
						term += ' type=' + this.typeSelect.value;
					}
					if (this.sortSelect.value !== '') {
						term += ' sort=' + this.sortSelect.value;
					}
					if (this.cropSelect.value !== '') {
						term += ' crop=' + this.cropSelect.value;
					}

					this.searchBox.set('value', $.trim(term));

					// submit search textbox
					this.searchBox.focus();
					this.invokeSearchManually = true;
					this.searchBox.onSearchBoxChange(this.searchBox.textbox.value);
				}.bind(this);
            },

            _onClearFilter: function () {                
                this.resetFilterSelection();                
                this.model._updateListQuery(this.model.selectedTreeItems);
            },

            getDigiZuiteConfigs: function () {
                when(this.searchStore.executeMethod('GetDigiZuiteSystemConfigs', ''),
                    function (response) {
                        this.bindSelectionControls(this.typeSelect, response.type);
                        this.bindSelectionControls(this.sortSelect, response.sort);
                        this.bindSelectionControls(this.cropSelect, response.crop);
                    }.bind(this));
            },

            bindSelectionControls: function (ctr, data) {
                ctr.set("options", array.map(data.data, function (item) {
                    return {
                        label: item.item2,
                        value: item.item1
                    };
                }, ctr));
                ctr.startup();
                ctr.dropDown.set('label', data.label);
                ctr.set('label', data.label);

                this.selectControls.push({ controlType: ctr, key: data.type, label: data.label });
            },

            resetFilterSelection: function () {
                if (this.typeSelect.select.value !== 4) {
                    domClass.toggle(this.cropSelect.domNode, "dijitHidden", false);
                }

                this.selectControls.forEach(function (item) {
                    item.controlType.value = undefined;
                    item.controlType.select.set('value', []);
                    item.controlType.select.set('checked', false);
                    item.controlType.dropDown.set('label', item.label);
                    item.controlType.select._updateSelection();
                });                
            }
		});
	});