define([
    "dojo/_base/declare",
    "dojo/io-query",
    "dijit/form/ToggleButton",
    "dijit/_WidgetBase",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/text!./templates/Selector.html"
], function (
    declare,
    ioQuery,
    ToggleButton,
    _WidgetBase,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    template
) {

    return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
        templateString: template,
        baseClass: "digi-mediaformat-selector",
        mediaFormats: null,
        image: null,

        buildRendering: function () {
            this.inherited(arguments);

            var selectedMediaFormat = this._getMediaFormatId(this.image);
            this.mediaFormats.forEach(function (item) {
                console.log(item);
                var button = new ToggleButton({ label: item.name, mediaFormatId: item.mediaFormatId, checked: item.mediaFormatId === selectedMediaFormat, iconClass: "epi-iconCheckmark", baseClass: "dijitToggleButton item" });

                this.own(button.on("change", this._onChange.bind(this, button)));
                this.own(button);
                button.placeAt(this._listNode);
            }, this);
        },

        executeDialog: function () {
            // Call this method to close the dialog
        },

        _onChange: function (button) {
            var checked = button.get("checked");
            var mediaFormatId = button.get("mediaFormatId");

            this.set("image", this._setFormatId(this.get("image"), checked ? mediaFormatId : null));

            this.executeDialog();
        },
        _getMediaFormatId: function (imgSrc) {
            var query = imgSrc.substring(imgSrc.indexOf("?") + 1, imgSrc.length);
            query = ioQuery.queryToObject(query);

            return query.mediaformatid ? parseInt(query.mediaformatid) : 0;
        },
        _setFormatId: function (imgSrc, selectedFormat) {
            var query = "";
            if (imgSrc.indexOf("?") !== -1) {
                query = imgSrc.substring(imgSrc.indexOf("?") + 1, imgSrc.length);
            }
            query = ioQuery.queryToObject(query);

            if (selectedFormat) {
                query.mediaformatid = selectedFormat;
            } else {
                delete query.mediaformatid;
            }

            if (imgSrc.indexOf("?") === -1) {
                return imgSrc + "?" + ioQuery.objectToQuery(query);
            } else {
                return imgSrc.substring(0, imgSrc.indexOf("?") + 1) + ioQuery.objectToQuery(query);
            }
        }
    });
});